import cProfile
import io
import pstats
import time


class Timing:
    def __init__(self, msg):
        self.msg = msg

    def __enter__(self):
        self.tini = time.time()

    def __exit__(self, *exc_info):
        tdelta = time.time() - self.tini
        print("======== {} {:.3f}s".format(self.msg, tdelta))


class Profile:
    def __init__(self, msg):
        self.msg = msg

    def __enter__(self):
        self.pr = cProfile.Profile()
        self.pr.enable()

    def __exit__(self, *exc_info):
        self.pr.disable()
        s = io.StringIO()
        ps = pstats.Stats(self.pr, stream=s).sort_stats('cumtime')
        ps.print_stats()
        print("========== profile {} start".format(self.msg))
        print(s.getvalue())
        print("========== profile {} end".format(self.msg))
